######################################################################
#:CQCC::IOClearPrompt 
#
# Licensed Materials - Property of IBM
# Rational ClearCase
# (C) Copyright IBM Corp. 2001, 2008. All Rights Reserved.
# US Government Users Restricted Rights
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#

=head1 NAME 

CQCC::IOClearPrompt - ClearPrompt subclass of IOBase for user and
system interaction methods

=head1 CLASS DESCRIPTION

The IOClearPrompt class is a subclass which provides an alternative
set of user interaction methods (IOPrompt, IOMessage, and
IOListSelect) that use the ClearPrompt GUI utility instead of a text
menu.

SUPPORT POLICY: See TriggerCQCC.pm for the Rational Support Policy. 

=head1 METHODS

=cut

######################################################################
#::Package declarations
######################################################################

package CQCC::IOClearPrompt;
use CQCC::IOBase;
@ISA = qw( CQCC::IOBase );
             
use strict;

use CQCC::Parm;
use CQCC::MsgCat_English;

sub Version { return '2.1' };

######################################################################
#::Class Globals 
######################################################################

my $logWarning = 0;

######################################################################
#::function ClassInit

=over 6

=item C<ClassInit()>

This class function is automatically invoked to define class 
parameters using CQCC::Parm objects. Currently there are no local
parameters. 

=back 

=cut

######################################################################

# sub ClassInit {}
# &ClassInit();   

######################################################################
#::function ClassTest

=over 6

=item C<ClassTest(@args)>

This class function sets IOBaseInterface to make an instance of 
IOClearPrompt then runs the class test for IOBase.

=back

=cut

######################################################################

sub ClassTest { 
    my @args = @_;
    &ForceParm("IOBaseInterface", "CQCC::IOClearPrompt");
    return &CQCC::IOBase::ClassTest(@args);
}

######################################################################
#::method new

=over 6

=item C<new CQCC::IOClearPrompt()>

This method initializes the object's attributes.

=back

=cut

######################################################################

sub new { 
   my $proto = shift;
   my $class = ref($proto) || $proto;  # Object or class name

   # Attributes
   my $self;

   # CCperl doesn't support SUPER so use direct reference to parent
   # IOBase is expected to complete initialization before returning
   # which should also preset file pathname suffixes, etc. 
   # Still can't call IO* functions until _IOStd is created and
   # we are still in the middle of that when this method is called. 

   # Prompt strings moved to MsgCat_English.pm

   $self = new CQCC::IOBase();
   $self->{CLASSNAME} = "CQCC::IOClearPrompt";
   bless($self, $class);
   return $self;
}

######################################################################
#::method Initialize

=over 6

=item C<Initialize()>

This method initializes the Clearprompt support files after the 
main base class has been initialized. Implicitly called by IOStd
when creating a new IOBase subclass object. 

=back

=cut

######################################################################

sub Initialize {
    my $self = shift;

    &CQCC::IOBase::Initialize($self);

    if ($self->{WINDOWS}) {
        $self->{CLEARPROMPT} = "clearprompt.exe";
        $ENV{"CLEARPROMPT_NEWLINE"} = 1;
    } else {
        $self->{CLEARPROMPT} = "clearprompt";
    } 
    $self->{PROMPT_FILE} = $self->Pathname("TEMP","iocprompt.txt");
    $self->{CPROMPT_DFILE} = 
        $self->Pathname("TEMP","iocprompt_dfile.txt");

    # Configure YES_TEXT and NO_TEXT for Web interface or standard Clearprompt GUI
    if ( defined $ENV{ATRIA_WEB_GUI} ) {
        &ForceParm("YES_TEXT", $self->{MSGCAT}->Message(["GUI_Commit"]));
        &ForceParm("NO_TEXT", $self->{MSGCAT}->Message(["GUI_Cancel"]));
    }
    else {
        &ForceParm("YES_TEXT", $self->{MSGCAT}->Message(["BUTTON_TEXT_YES"]));
        &ForceParm("NO_TEXT", $self->{MSGCAT}->Message(["BUTTON_TEXT_ABORT"]));
    }
}

######################################################################
#::method ListSelect

=over 6

=item C<ListSelect(multiChoice, titleMsgTag, choiceMsgTagList...)>

This method presents a menu with a title described by "titleMsgTag"
and one or more options to select from described by the remaining
arguments. If multiChoice is 1 then multiple choices are allowed, 
otherwise only one choice is provided. 

In the ClearPrompt implementation, the choices are displayed in a
scolling "list" type menu and the user is expected to pick one or more
lines in the list.

The selected items will be returned as a list to the caller, 
using the original choiceMsgTag items in their entirety for the caller
to match to expectations. 

The return value consists of a list containing the a flag to abort
(1 for abort, 0 for continue) and the answer(s).

=back 

=cut

######################################################################

sub ListSelect {
    my ($self, $multiChoice, $titleMsgTag, @choices) = @_;

    my (@answers,$i,$done,$choice, $msg, $menuLine,$maxItems);
    my (@displayChoices, $indexedAnswer, $abort);
    
    $self->_LogEntry("ListSelect", $titleMsgTag);

    $msg = $self->{MSGCAT}->Message($titleMsgTag);

    $indexedAnswer = $self->_ReadSTDIN( "BEFORE",
           ["IO_P_LISTSELECT", ($multiChoice ? "(s)" : "")]);


    ##################################################
    # Prep choices for ClearPrompt, handle user interaction, 
    # then translate answers into numbers for recording to journal. 

    if (! defined $indexedAnswer) { 

        # Translate messages and remove extra commas because 
        # Clearprompt uses them as delimiters. 

        for $choice (@choices) {
            $menuLine = $self->{MSGCAT}->Message($choice);
            $menuLine =~ s/,/ /g;
            push @displayChoices, $menuLine;
        }

        &IOFileWrite($self->{CPROMPT_DFILE}, 
            join(",\n", @displayChoices) . "\n");

        ($abort, @answers) = $self->_ClearPrompt("list -prefer_gui" .
            " -dfile " . $self->{CPROMPT_DFILE} .
            ($multiChoice ? " -choices" : "") . 
            " -prompt \"" . $msg . "\"");

        # Need to translate answers into numbers on a single string
        # in order to store the answer and need to translate the 
        # modified choice strings (stripped commas, etc) to match 
        # original choices.
        # Return the original choices list not the modified version.
        # Strip trailing spaces off both the answer and the test
        #   strings because ClearPrompt may strip off spaces

        my $numberString = "";
        my (@realAnswers, $displayAnswer);

        for $displayAnswer (@answers) { 
            my $i = 1;
            $displayAnswer =~ s/\s+$//;
            for (@displayChoices) {
                my $choice = $_;
                $choice =~ s/\s+$//;
                if ($displayAnswer eq $choice) { 
                   push @realAnswers, $choices[$i-1];
                   $numberString .= "$i ";
                   last;
                }
                $i++;
            } 
        }
        @answers = @realAnswers;
        $indexedAnswer = $numberString;

    ##################################################
    # If log file provided answer string then translate numbers 
    # back into choices (without comma stripping, etc)

    } else { 
        my @numberedChoices = &IOParseListString($indexedAnswer);
        my @realAnswers;
        
        for (@numberedChoices) {
            my $index = $_;   # Note: indices are 1-based
            if (($index > 0) && ($index <= $#choices)) {
                push @realAnswers, $choices[$index-1];
            }
            else {
                &IOPrint("Error: IOClearPrompt::ListSelect journal " .
                    "answer list index ($index) out of range\n");
            }
        }
    }

    # Save the numbered string so that the text driven playback 
    # can work with the IOBase version of ListSelect().

    $self->_ReadSTDIN( "AFTER",
        ["IO_P_LISTSELECT", ($multiChoice ? "(s)" : "")],
        $indexedAnswer);

    return ($abort, @answers);
}

######################################################################
#::method Message

=over 6

=item C<Message(type, abort, [msgtag, args])>

This method (called through IOMessage()) will display its message
to the user, after translating the msgtag and its arguments into the
corresponding message catalog entry. Display is handled by calling
the Prompt() method. 

The type value may be "INFO", "WARN", "ERROR", or "FATAL" and is
intended to qualify how the message is to be shown and whether or
not the message affects the overall trigger status. If "FATAL" the
status will be set to failed and the trigger will try to cleanly
unwind and report failure back to the original caller, usually
ClearCase.

The abort flag is also used to signal general failure for terminal
conditions and will be combined with the type information shortly. 

=back 

=cut

######################################################################

sub Message {
    my ($self, $type, $msgTag) = @_;
    my ($package, $filename, $line) = caller 1;
    my $proceed = 1;
    my $answer;
    my $maxChars = 1400;
    if (! defined $msgTag) {
        &IOMessage("ERROR", ["IO_E_MSGTAG_UNDEF"]);
        IOTraceBack("Message", 10);
        return;
    }

    my $msgType = $self->{MSGCAT}->Message([$type]);
    my $msg = $self->{MSGCAT}->Message($msgTag) . "\n";
    $self->_LogEntry("Message", $msgTag);
    
    my $displayMsg = $msgType . ": " . $msg;

    if ($type eq "ERROR" || $type eq "FATAL") {
        $displayMsg .= sprintf "Caller: $filename Line: $line\n";
    }

    # Working around a bug in ClearPrompt that doesn't work properly
    # with newlines. For now mush all the lines together in wrap mode
    # remove double spaces and try to make it work as a new paragraph

    my $lineSep = "\n";
    if ($self->{WINDOWS}) { 
        $lineSep = "\\n";
        $displayMsg =~ s/  / /g;
        $displayMsg =~ s/  / /g;
    } 

    my @displayList = split("\n", $displayMsg);
    $displayMsg = "";
    my $i;
    my $lineCount = $#displayList;
    for ($i = 0; $i <= $lineCount; $i++) {
        $displayMsg .= $displayList[$i] . $lineSep;
        if (($i == $lineCount) ||
            (length($displayMsg) + length($displayList[$i+1] + 2) > $maxChars)) 
        {
            $answer = $self->_ReadSTDIN("BEFORE", ["IO_P_MESSAGE" ]);
            if (! defined $answer) { 
                $self->_ClearPrompt("proceed -prefer_gui -mask Proceed " . 
                    "-prompt \"$displayMsg\"");
            }    
            $self->_ReadSTDIN("AFTER", ["IO_P_MESSAGE" ], $answer);
            $displayMsg = ""; # Clear page once shown
        }
    }

    # Record message to error log if enabled
    $self->MessageLog($type,$msg,$filename,$line);

    if ($type eq "FATAL") {
        &IOExitHandler(2);
    } elsif ($proceed == 0) {
        &IOExitHandler(1);
    }

    return;
}

######################################################################
#::method Print

=over 6 

=item C<Print(print-args)>

This method handles print operations, allowing prints to be 
redirected to a log as well as stdout. 

=back 

=cut

######################################################################

sub Print{
    my ($self, @args) = @_;

    $self->PrintLog($IO_FILE_APPEND,@args);

    if (! $logWarning) { 
        printf "Debug and timing information printed to log file %s\n",
            $self->{LOG_FILE};
        $logWarning = 1;
    } 
    # Don't print to window, just to log file
    # print @args;
}

######################################################################
#::method Prompt

=over 6

=item C<Prompt(promptMsg, optionsList)>

This internal function displays its message to the user, after
translating the msgtag and its arguments into the corresponding
message catalog entry. In addition to the required message, it 
accepts optional qualifiers in the form of "option=>value" pairs to
support multiple public API calls like IOPromptLogin.

Available options include the following: 

   "IOPROMPT_MODE" =>     # Prompt display mode, one of the following

       "IOPROMPT_TEXT"   # Default, display message, expect user input 

  or   "IOPROMPT_LOGIN"  # Login prompt, expect user to enter username and (masked) password

  or   "IOPROMPT_YES_NO" # Present user choice of yes or no input

   "IOPROMPT_LOGIN_MSG"=>loginPromptMessage  # Alternate login prompt

The return value consists of a list containing the abort status
(1 for abort, 0 for continue) and the answer(s). 

"IOPROMPT_LOGIN" returns two strings, username and password.
"IOPROMPT_YES_NO" returns a single integer, 1 for yes, 0 for no.

($abort, $answer(s)) = $obj->Prompt(msg, option1=>value1, ... );

=back

=cut

######################################################################

sub Prompt {
    my ($self, $msgTag, %options) = @_;
    my $mode = (defined $options{"IOPROMPT_MODE"}) ? 
               $options{"IOPROMPT_MODE"} : "IOPROMPT_TEXT";
    my $clearpromptMode = 
        ($mode eq "IOPROMPT_YES_NO") ? "yes_no" : "text";
    my ($answer,$abort);
    my $msg = $self->{MSGCAT}->Message($msgTag);

    $self->_LogEntry("Prompt", $msgTag);

    $answer = $self->_ReadSTDIN("BEFORE", ["IO_P_MESSAGE" ]);
    if ($self->{WINDOWS}) { 
        $msg =~ s/\n/\\n/g;
    } 
    ($abort, $answer) = 
        $self->_ClearPrompt("$clearpromptMode -prefer_gui " . 
            "-prompt \"$msg\"");
    $self->_ReadSTDIN("AFTER", ["IO_P_MESSAGE" ], $answer);
    if ($mode eq "IOPROMPT_LOGIN") {
        my ($username,$password) = split(/[\s,;:]/, $answer, 2);
        return ($abort, $username, $password);
    }
    return ($abort, $answer);
}

######################################################################
#::method _ClearPrompt

=over 6

=item C<_ClearPrompt(command)>

This method will finish formatting the ClearPrompt command string 
and execute it. On the PC it writes out the ClearPrompt command
to a file to workaround known bugs. 

The return value consists of a list containing a flag to abort
(1 for abort, 0 for continue) and the answer(s).

($abort, $answer) = $obj->Prompt(keys);

=back

=cut

######################################################################

sub _ClearPrompt { 
    my ($self, $cmd, $outfile) = @_;
    my (@answer, $result);

    my $cp_cmd = $self->{CLEARPROMPT} . " " . $cmd;

    if (substr($cmd,0,5) eq "list " ||
        substr($cmd,0,5) eq "text ") { 
        $cp_cmd .= " -outfile $self->{PROMPT_FILE}";
        $outfile = 1;
    }

    $self->DebugPrint("DETAILED",$cp_cmd);

    # Work around clearprompt problems on PC - seems to 
    # work best using indirect batch file rather than backticks

    if ($self->{WINDOWS}) {
        my $batname = $self->Pathname("TEMP","cqcc_prompt.bat");
        $self->FileWrite($batname,
           ((&GetParm("CQCC_DEBUG") > 1) ? '@echo on' : '@echo off'),
           $cp_cmd);
        $result = system($batname);
        if ($result != 0) { 
            IODebugPrint("BASIC", "ClearPrompt result: $result\n"); 
        }
        unlink $batname;
    } else {
        $result = system($cp_cmd);
    }

    if ($result == 0 && $outfile) {
        my $r;
        @answer = $self->FileRead($self->{PROMPT_FILE}, 1);
    }
    if (substr($cmd,0,7) eq "yes_no ") {
        # Yes
        if ($result eq 0) {
            return(0, 1);

        # No
        } elsif ($result eq 1 || $result eq 256) {
            return(0, 0);

        # Cancel covers multiple return codes
        } else {
            return(1, 0);
        }
    }

    return ($result ? 1 : 0, @answer);
}

######################################################################
#::Tail
######################################################################

1;
